using Microsoft.AspNetCore.Mvc;

namespace MyApp.Namespace
{
    [Route("[controller]")]
    [ApiController]
    public class ExperienceController : ControllerBase
    {
        private readonly IExperienceService experienceService;
        private readonly IProviderService providerService;

        public ExperienceController(IExperienceService experienceService, IProviderService providerService)
        {
            this.experienceService = experienceService;
            this.providerService = providerService;
        }

        [HttpGet(Name = "ExperiencesGet")]
        public async Task<IActionResult> Get()
        {
            var result = await experienceService.GetExperiences();
            if (result == null)
            {
                return NotFound();
            }
            return Ok(result);
        }

        [HttpGet("{name}", Name = "ExperienceGet")]
        public async Task<IActionResult> Get(string name)
        {
            var result = await experienceService.GetExperience(name);
            if (result == null)
            {
                return NotFound();
            }
            return Ok(result);
        }

        [HttpGet("search")]
        public async Task<IActionResult> Search(
            [FromQuery] string? name,
            [FromQuery] string? description,
            [FromQuery] decimal? price)
        {
            var result = await experienceService.SearchExperience(name, description, price);
            if (result == null)
            {
                return NotFound();
            }
            return Ok(result);
        }

        [HttpGet("statistics")]
        public async Task<IActionResult> GetStatistics()
        {
            var result = await experienceService.GetStatistics();
            if (result == null)
            {
                return NotFound();
            }
            return Ok(result);
        }

        [HttpGet("sales")]
        public async Task<IActionResult> GetSales()
        {
            var result = await experienceService.GetSales();
            if (result == null)
            {
                return NotFound();
            }
            return Ok(result);
        }

        [HttpPost(Name = "ExperiencePost")]
        public async Task<IActionResult> Post([FromBody] CreateExperienceModel experience)
        {
            if (!ModelState.IsValid)
            {
                return BadRequest(ModelState);
            }
            var result = await experienceService.CreateExperience(experience);

            if (result == null)
            {
                return BadRequest("Could not create experience, provider may be non existing.");
            }

            return CreatedAtRoute("ExperienceGet", new { name = result.Name }, result);
        }

        [HttpPut("{id}")]
        public async Task<IActionResult> UpdatePrice(int id, decimal price)
        {
            var result = await experienceService.UpdateExperience(id, price);
            if (result == null)
            {
                return NotFound();
            }
            return Ok(result);
        }

        [HttpDelete("{id}")]
        public async Task<IActionResult> Delete(int id)
        {
            var result = await experienceService.DeleteExperience(id);
            if (result == null)
            {
                return NotFound();
            }
            return Ok(result);
        }
    }
}
